#ifndef __RTL_COMPATIBILITY__
#define __RTL_COMPATIBILITY__

/* compatibility */

#include <linux/malloc.h>

typedef long long RTIME; /* 8254 ticks */
#define RT_TICKS_PER_SEC 1193180LL
#define RT_TIME_END 0x7fffFfffFfffFfffLL

extern long scaler_8254_to_ns;
extern long scaler_ns_to_8254;
extern inline struct timespec RTIME_to_timespec (RTIME t)
{
	long long ns;
	__asm__(
		"shld $10, %%eax, %%edx\n\t"
		"shl $10, %%eax\n\t"
		"mov %%edx, %%ecx\n\t"
		"mul %%ebx\n\t"  	/* multiply the low 32 bits of the counter by the scaler */
		"mov %%ecx, %%eax\n\t"
		"mov %%edx, %%ecx\n\t"	/* save the high 32 bits of the product */
		"mul %%ebx\n\t" 	/* now the high 32 bits */
		"add %%ecx, %%eax\n\t"
		"adc $0, %%edx\n\t"
		:"=A" (ns) : "b" (scaler_8254_to_ns), "A" (t): "cx");
	return timespec_from_ns(ns);
}

extern inline RTIME timespec_to_RTIME (struct timespec ts)
{
	long long ns = timespec_to_ns (&ts);
	RTIME t = timespec_to_ns (&ts);
	__asm__("mov %%edx, %%ecx\n\t"
		"mull %%ebx\n\t"
		"mov %%ecx, %%eax\n\t"
		"mov %%edx, %%ecx\n\t"	/* save the high 32 bits of the product */
		"mul %%ebx\n\t" 	/* now the high 32 bits */
		"add %%ecx, %%eax\n\t"
		"adc $0, %%edx\n\t"
		:"=A" (t) :"A" (ns), "b" (scaler_ns_to_8254)
		:"cx");
	return t;
}


extern inline RTIME rt_get_time(void)
{
	struct timespec ts;
	clock_gettime(CLOCK_REALTIME, &ts);
	return timespec_to_RTIME(ts);
}

extern inline RTIME rtl_set_periodic_mode (RTIME period)
{
	struct timespec ts;
	int ret;
	DECLARE_CPUID(cpu_id);
	ts = timespec_from_ns (period);
	ret = rtl_clock_set_periodic_mode (rtl_clock_local(cpu_id), &ts);
	if (ret == 0) {
		clock_gettime(rtl_clock_local(cpu_id), &ts);
		return timespec_to_ns(&ts);
	} else {
		return RT_TIME_END;
	}
}

extern inline int rtl_set_oneshot_mode (void)
{
	DECLARE_CPUID(cpu_id);
	return rtl_clock_set_oneshot_mode(rtl_clock_local(cpu_id));
}



extern inline int rt_task_use_fp (pthread_t *task, int flag) {
	return rtl_pthread_use_fp (*task, flag);
}

/* cross-CPU-allowed functions */
extern int rt_task_suspend(pthread_t *task);
extern int rt_task_wakeup(pthread_t *thread);


extern inline int rt_task_delete(pthread_t *thread)
{
	int ret;
	ret = rtl_pthread_delete(*thread);
	timer_delete((*thread)->timer);
	return 0;
}

extern int rt_task_make_periodic (pthread_t *threadptr, RTIME start_time, RTIME period);

/* end of cross-CPU-allowed functions */


/* for periodic tasks -- wait until the next period */
extern inline int rt_task_wait (void)
{
	rtl_pthread_wait_timer(pthread_self()->timer);
	return 0;
}


#define RT_TASK pthread_t


extern int rt_task_init (RT_TASK *t, void (*fn)(int data), int data, int stack_size, int priority);

#define rtl_init (a,b,c,d,e) { rt_task_init(a,b,c,d,e); }
#define rtl_fpinit (a,b,c,d,e) { rt_task_init(a,b,c,d,e); rtl_task_use_fp (a, 1); }

#endif
